FEATURE_TAGS = {
    "aalt": {"name": "Access All Alternates"},
    "abvf": {"name": "Above-base Forms"},
    "abvm": {"name": "Above-base Mark Positioning"},
    "abvs": {"name": "Above-base Substitutions"},
    "afrc": {"name": "Alternative Fractions"},
    "akhn": {"name": "Akhand"},
    "blwf": {"name": "Below-base Forms"},
    "blwm": {"name": "Below-base Mark Positioning"},
    "blws": {"name": "Below-base Substitutions"},
    "c2pc": {"name": "Petite Capitals From Capitals"},
    "c2sc": {"name": "Small Capitals From Capitals"},
    "calt": {"name": "Contextual Alternates"},
    "case": {"name": "Case-Sensitive Forms"},
    "ccmp": {"name": "Glyph Composition/Decomposition"},
    "cfar": {"name": "Conjunct Form After Ro"},
    "chws": {"name": "Contextual Half-width Spacing"},
    "cjct": {"name": "Conjunct Forms"},
    "clig": {"name": "Contextual Ligatures"},
    "cpct": {"name": "Centered CJK Punctuation"},
    "cpsp": {"name": "Capital Spacing"},
    "cswh": {"name": "Contextual Swash"},
    "curs": {"name": "Cursive Positioning"},
    "dist": {"name": "Distances"},
    "dlig": {"name": "Discretionary Ligatures"},
    "dnom": {"name": "Denominators"},
    "dtls": {"name": "Dotless Forms"},
    "expt": {"name": "Expert Forms"},
    "falt": {"name": "Final Glyph on Line Alternates"},
    "fin2": {"name": "Terminal Form #2"},
    "fin3": {"name": "Terminal Form #3"},
    "fina": {"name": "Terminal Forms"},
    "flac": {"name": "Flattened accent forms"},
    "frac": {"name": "Fractions"},
    "fwid": {"name": "Quarter Widths"},
    "half": {"name": "Half Forms"},
    "haln": {"name": "Halant Forms"},
    "halt": {"name": "Alternate Half Widths"},
    "hist": {"name": "Historical Forms"},
    "hkna": {"name": "Horizontal Kana Alternates"},
    "hlig": {"name": "Historical Ligatures"},
    "hngl": {"name": "Hangul"},
    "hojo": {"name": "Hojo Kanji Forms (JIS X 0212-1990 Kanji Forms)"},
    "hwid": {"name": "Half Widths"},
    "init": {"name": "Initial Forms"},
    "isol": {"name": "Isolated Forms"},
    "ital": {"name": "Italics"},
    "jalt": {"name": "Justification Alternates"},
    "jp04": {"name": "JIS04 Forms"},
    "jp78": {"name": "JIS78 Forms"},
    "jp83": {"name": "JIS83 Forms"},
    "jp90": {"name": "JIS90 Forms"},
    "kern": {"name": "Kerning"},
    "lfbd": {"name": "Left Bounds"},
    "liga": {"name": "Standard Ligatures"},
    "ljmo": {"name": "Leading Jamo Forms"},
    "lnum": {"name": "Lining Figures"},
    "locl": {"name": "Localized Forms"},
    "ltra": {"name": "Left-to-right glyph alternates"},
    "ltrm": {"name": "Left-to-right mirrored forms"},
    "mark": {"name": "Mark Positioning"},
    "med2": {"name": "Medial Forms #2"},
    "medi": {"name": "Medial Forms"},
    "mgrk": {"name": "Mathematical Greek"},
    "mkmk": {"name": "Mark to Mark Positioning"},
    "mset": {"name": "Mark Positioning via substitution"},
    "nalt": {"name": "Alternate Annotation Forms"},
    "nlck": {"name": "NLC Kanji Forms"},
    "nukt": {"name": "Nukta Forms"},
    "numr": {"name": "Numerators"},
    "onum": {"name": "Oldstyle Figures"},
    "opbd": {"name": "Optical Bounds"},
    "ordn": {"name": "Ordinals"},
    "ornm": {"name": "Ornaments"},
    "palt": {"name": "Proportional Alternate Widths"},
    "pcap": {"name": "Petite Capitals"},
    "pkna": {"name": "Proportional Kana"},
    "pnum": {"name": "Proportional Figures"},
    "pref": {"name": "Pre-base Forms"},
    "pres": {"name": "Pre-base Substitutions"},
    "pstf": {"name": "Post-base Forms"},
    "psts": {"name": "Post-base Substitutions"},
    "pwid": {"name": "Proportional Widths"},
    "qwid": {"name": "Quarter Widths"},
    "rand": {"name": "Randomize"},
    "rclt": {"name": "Required Contextual Alternates"},
    "rkrf": {"name": "Rakar Forms"},
    "rlig": {"name": "Required Ligatures"},
    "rphf": {"name": "Reph Form"},
    "rtbd": {"name": "Right Bounds"},
    "rtla": {"name": "Right-to-left alternates"},
    "rtlm": {"name": "Right-to-left mirrored forms"},
    "ruby": {"name": "Ruby Notation Forms"},
    "rvrn": {"name": "Required Variation Alternates"},
    "salt": {"name": "Stylistic Alternates"},
    "sinf": {"name": "Scientific Inferiors"},
    "size": {"name": "Optical size"},
    "smcp": {"name": "Small Capitals"},
    "smpl": {"name": "Simplified Forms"},
    "ssty": {"name": "Math script style alternates"},
    "stch": {"name": "Stretching Glyph Decomposition"},
    "subs": {"name": "Subscript"},
    "sups": {"name": "Superscript"},
    "swsh": {"name": "Swash"},
    "titl": {"name": "Titling"},
    "tjmo": {"name": "Trailing Jamo Forms"},
    "tnam": {"name": "Traditional Name Forms"},
    "tnum": {"name": "Tabular Figures"},
    "trad": {"name": "Traditional Forms"},
    "twid": {"name": "Third Widths"},
    "unic": {"name": "Unicase"},
    "valt": {"name": "Alternate Vertical Metrics"},
    "vatu": {"name": "Vattu Variants"},
    "vchw": {"name": "Vertical Contextual Half-width Spacing"},
    "vert": {"name": "Vertical Alternates"},
    "vhal": {"name": "Alternate Vertical Half Metrics"},
    "vjmo": {"name": "Vowel Jamo Forms"},
    "vkna": {"name": "Vertical Kana Alternates"},
    "vkrn": {"name": "Vertical Kerning"},
    "vpal": {"name": "Proportional Alternate Vertical Metrics"},
    "vrt2": {"name": "Vertical Alternates and Rotation"},
    "vrtr": {"name": "Vertical Alternates for Rotation"},
    "zero": {"name": "Slashed Zero"},
}

for r in range(1, 100):
    FEATURE_TAGS["cv%02i" % r] = {"name": "Character variant %i" % r}
for r in range(1, 21):
    FEATURE_TAGS["ss%02i" % r] = {"name": "Stylistic Set %i" % r}



SCRIPT_TAGS = {'DFLT': {'name': 'Default'},
 'adlm': {'name': 'Adlam'},
 'aghb': {'name': 'Caucasian Albanian'},
 'ahom': {'name': 'Ahom'},
 'arab': {'name': 'Arabic'},
 'armi': {'name': 'Imperial Aramaic'},
 'armn': {'name': 'Armenian'},
 'avst': {'name': 'Avestan'},
 'bali': {'name': 'Balinese'},
 'bamu': {'name': 'Bamum'},
 'bass': {'name': 'Bassa Vah'},
 'batk': {'name': 'Batak'},
 'beng': {'name': 'Bangla'},
 'bhks': {'name': 'Bhaiksuki'},
 'bng2': {'name': 'Bangla v.2'},
 'bopo': {'name': 'Bopomofo'},
 'brah': {'name': 'Brahmi'},
 'brai': {'name': 'Braille'},
 'bugi': {'name': 'Buginese'},
 'buhd': {'name': 'Buhid'},
 'byzm': {'name': 'Byzantine Music'},
 'cakm': {'name': 'Chakma'},
 'cans': {'name': 'Canadian Syllabics'},
 'cari': {'name': 'Carian'},
 'cham': {'name': 'Cham'},
 'cher': {'name': 'Cherokee'},
 'chrs': {'name': 'Chorasmian'},
 'copt': {'name': 'Coptic'},
 'cpmn': {'name': 'Cypro-Minoan'},
 'cprt': {'name': 'Cypriot Syllabary'},
 'cyrl': {'name': 'Cyrillic'},
 'dev2': {'name': 'Devanagari v.2'},
 'deva': {'name': 'Devanagari'},
 'diak': {'name': 'Dives Akuru'},
 'dogr': {'name': 'Dogra'},
 'dsrt': {'name': 'Deseret'},
 'dupl': {'name': 'Duployan'},
 'egyp': {'name': 'Egyptian Hieroglyphs'},
 'elba': {'name': 'Elbasan'},
 'elym': {'name': 'Elymaic'},
 'ethi': {'name': 'Ethiopic'},
 'gara': {'name': 'Garay'},
 'geor': {'name': 'Georgian'},
 'gjr2': {'name': 'Gujarati v.2'},
 'glag': {'name': 'Glagolitic'},
 'gong': {'name': 'Gunjala Gondi'},
 'gonm': {'name': 'Masaram Gondi'},
 'goth': {'name': 'Gothic'},
 'gran': {'name': 'Grantha'},
 'grek': {'name': 'Greek'},
 'gujr': {'name': 'Gujarati'},
 'gukh': {'name': 'Gurung Khema'},
 'gur2': {'name': 'Gurmukhi v.2'},
 'guru': {'name': 'Gurmukhi'},
 'hang': {'name': 'Hangul'},
 'hani': {'name': 'CJK Ideographic'},
 'hano': {'name': 'Hanunoo'},
 'hatr': {'name': 'Hatran'},
 'hebr': {'name': 'Hebrew'},
 'hluw': {'name': 'Anatolian Hieroglyphs'},
 'hmng': {'name': 'Pahawh Hmong'},
 'hmnp': {'name': 'Nyiakeng Puachue Hmong'},
 'hung': {'name': 'Old Hungarian'},
 'ital': {'name': 'Old Italic'},
 'jamo': {'name': 'Hangul Jamo'},
 'java': {'name': 'Javanese'},
 'kali': {'name': 'Kayah Li'},
 'kana': {'name': 'Katakana'},
 'kawi': {'name': 'Kawi'},
 'khar': {'name': 'Kharosthi'},
 'khmr': {'name': 'Khmer'},
 'khoj': {'name': 'Khojki'},
 'kits': {'name': 'Khitan Small Script'},
 'knd2': {'name': 'Kannada v.2'},
 'knda': {'name': 'Kannada'},
 'krai': {'name': 'Kirat Rai'},
 'kthi': {'name': 'Kaithi'},
 'lana': {'name': 'Tai Tham (Lanna)'},
 'lao ': {'name': 'Lao'},
 'latn': {'name': 'Latin'},
 'lepc': {'name': 'Lepcha'},
 'limb': {'name': 'Limbu'},
 'lina': {'name': 'Linear A'},
 'linb': {'name': 'Linear B'},
 'lisu': {'name': 'Lisu (Fraser)'},
 'lyci': {'name': 'Lycian'},
 'lydi': {'name': 'Lydian'},
 'mahj': {'name': 'Mahajani'},
 'maka': {'name': 'Makasar'},
 'mand': {'name': 'Mandaic, Mandaean'},
 'mani': {'name': 'Manichaean'},
 'marc': {'name': 'Marchen'},
 'math': {'name': 'Mathematical text layout'},
 'medf': {'name': 'Medefaidrin (Oberi Okaime, Oberi Ɔkaimɛ)'},
 'mend': {'name': 'Mende Kikakui'},
 'merc': {'name': 'Meroitic Cursive'},
 'mero': {'name': 'Meroitic Hieroglyphs'},
 'mlm2': {'name': 'Malayalam v.2'},
 'mlym': {'name': 'Malayalam'},
 'modi': {'name': 'Modi'},
 'mong': {'name': 'Mongolian'},
 'mroo': {'name': 'Mro'},
 'mtei': {'name': 'Meitei Mayek (Meithei, Meetei)'},
 'mult': {'name': 'Multani'},
 'musc': {'name': 'Musical Symbols'},
 'mym2': {'name': 'Myanmar v.2'},
 'mymr': {'name': 'Myanmar'},
 'nagm': {'name': 'Nag Mundari'},
 'nand': {'name': 'Nandinagari'},
 'narb': {'name': 'Old North Arabian'},
 'nbat': {'name': 'Nabataean'},
 'newa': {'name': 'Newa'},
 'nko ': {'name': "N'Ko"},
 'nshu': {'name': 'Nüshu'},
 'ogam': {'name': 'Ogham'},
 'olck': {'name': 'Ol Chiki'},
 'onao': {'name': 'Ol Onal'},
 'orkh': {'name': 'Old Turkic, Orkhon Runic'},
 'ory2': {'name': 'Odia v.2'},
 'orya': {'name': 'Odia'},
 'osge': {'name': 'Osage'},
 'osma': {'name': 'Osmanya'},
 'ougr': {'name': 'Old Uyghur'},
 'palm': {'name': 'Palmyrene'},
 'pauc': {'name': 'Pau Cin Hau'},
 'perm': {'name': 'Old Permic'},
 'phag': {'name': 'Phags-pa'},
 'phli': {'name': 'Inscriptional Pahlavi'},
 'phlp': {'name': 'Psalter Pahlavi'},
 'phnx': {'name': 'Phoenician'},
 'plrd': {'name': 'Miao'},
 'prti': {'name': 'Inscriptional Parthian'},
 'rjng': {'name': 'Rejang'},
 'rohg': {'name': 'Hanifi Rohingya'},
 'runr': {'name': 'Runic'},
 'samr': {'name': 'Samaritan'},
 'sarb': {'name': 'Old South Arabian'},
 'saur': {'name': 'Saurashtra'},
 'sgnw': {'name': 'Sign Writing'},
 'shaw': {'name': 'Shavian'},
 'shrd': {'name': 'Sharada'},
 'sidd': {'name': 'Siddham'},
 'sind': {'name': 'Khudawadi'},
 'sinh': {'name': 'Sinhala'},
 'sogd': {'name': 'Sogdian'},
 'sogo': {'name': 'Old Sogdian'},
 'sora': {'name': 'Sora Sompeng'},
 'soyo': {'name': 'Soyombo'},
 'sund': {'name': 'Sundanese'},
 'sunu': {'name': 'Sunuwar'},
 'sylo': {'name': 'Syloti Nagri'},
 'syrc': {'name': 'Syriac'},
 'tagb': {'name': 'Tagbanwa'},
 'takr': {'name': 'Takri'},
 'tale': {'name': 'Tai Le'},
 'talu': {'name': 'New Tai Lue'},
 'taml': {'name': 'Tamil'},
 'tang': {'name': 'Tangut'},
 'tavt': {'name': 'Tai Viet'},
 'tel2': {'name': 'Telugu v.2'},
 'telu': {'name': 'Telugu'},
 'tfng': {'name': 'Tifinagh'},
 'tglg': {'name': 'Tagalog'},
 'thaa': {'name': 'Thaana'},
 'thai': {'name': 'Thai'},
 'tibt': {'name': 'Tibetan'},
 'tirh': {'name': 'Tirhuta'},
 'tml2': {'name': 'Tamil v.2'},
 'tnsa': {'name': 'Tangsa'},
 'todr': {'name': 'Todhri'},
 'toto': {'name': 'Toto'},
 'tutg': {'name': 'Tulu-Tigalari'},
 'ugar': {'name': 'Ugaritic Cuneiform'},
 'vai ': {'name': 'Vai'},
 'vith': {'name': 'Vithkuqi'},
 'wara': {'name': 'Warang Citi'},
 'wcho': {'name': 'Wancho'},
 'xpeo': {'name': 'Old Persian Cuneiform'},
 'xsux': {'name': 'Sumero-Akkadian Cuneiform'},
 'yezi': {'name': 'Yezidi'},
 'yi  ': {'name': 'Yi'},
 'zanb': {'name': 'Zanabazar Square (Zanabazarin Dörböljin Useg, Xewtee '
                  'Dörböljin Bicig, Horizontal Square Script)'}}



from bs4 import BeautifulSoup
import requests
from pprint import pprint
import re

script_tags_url = "https://learn.microsoft.com/en-gb/typography/opentype/spec/scripttags"
lang_tags_url = "https://learn.microsoft.com/en-gb/typography/opentype/spec/languagetags"

r = requests.get(script_tags_url)
r.encoding = r.apparent_encoding
soup = BeautifulSoup(r.text, 'html.parser')
table = soup.find("table")
SCRIPT_TAGS = {}
for row in table.find_all("tr"):
    columns = row.find_all("td")
    if len(columns) == 3:
        name, tag, description = columns
        m = re.search("'([A-Za-z0-9\s]{4})'", tag.text.strip())
        if m:
            tag = m.group(1)
        else:
            tag = tag.text.strip()
        tag = tag.replace("\xa0", " ") # Yi!
        SCRIPT_TAGS[tag] = {"name": name.text.strip()}

r = requests.get(lang_tags_url)
r.encoding = r.apparent_encoding
soup = BeautifulSoup(r.text, 'html.parser')
table = soup.find("table")
LANGUAGE_TAGS = {}
for row in table.find_all("tr"):
    columns = row.find_all("td")
    if len(columns) == 3:
        name, tag, iso639 = columns
        iso639 = re.sub(r"See.*", "", iso639.text.strip()).split(", ") # Mon
        if not iso639[0]:
            iso639 = []
        m = re.search("'([A-Za-z0-9\s]{4})'", tag.text.strip())
        if m:
            tag = m.group(1)
        else:
            tag = tag.text.strip()
        LANGUAGE_TAGS[tag] = {
            "name": name.text.strip(),
            "iso639": iso639
        }

with open("opentypespec/tags/scripts.py", "w") as fh:
    fh.write("SCRIPT_TAGS = ")
    pprint(SCRIPT_TAGS, fh)

with open("opentypespec/tags/languages.py", "w") as fh:
    fh.write("LANGUAGE_TAGS = ")
    pprint(LANGUAGE_TAGS, fh)